// $Id: popups.js,v 1.9.8.7 2009/03/16 06:24:02 starbow Exp $

/**
 * Popup Modal Dialog API
 *
 * Provide an API for building and displaying JavaScript, in-page, popups modal dialogs.
 * Modality is provided by a fixed, semi-opaque div, positioned in front of the page contents.
 *
 */

/*
 * TODO
 * * Return key in add node form not working.
 * * Tabledrag breaking after ahah reload.
 */

// ***************************************************************************
// DRUPAL Namespace
// ***************************************************************************

/**
 * Attach the popups bevior to the all the requested links on the page.
 *
 * @param context
 *   The jQuery object to apply the behaviors to.
 */

Drupal.behaviors.popups = function(context) {
  var $body = $('body');
  if(!$body.hasClass('popups-processed')) {
    $body.addClass('popups-processed');
    $(document).bind('keydown', Popups.keyHandle);
    var $popit = $('#popit');
    if ($popit.length) {
      $popit.remove();
      Popups.message($popit.html());
    }
  }
  
  // Add the popups-link-in-dialog behavior to links defined in Drupal.settings.popups.links array.
  if (Drupal.settings.popups.links) {
    jQuery.each(Drupal.settings.popups.links, function (link, options) { 
      if (Popups.isset(options.noReload)) { // Using obsolete name.
        options.noUpdate = options.noReload; // Don't break existing sites with name change.
      }
      Popups.attach(context, link, options);
    });
  }
  
  Popups.attach(context, '.popups', {noUpdate: true});  
  Popups.attach(context, '.popups-form', {}); // ajax reload.
  Popups.attach(context, '.popups-form-reload', {reloadWhenDone: true}); // whole page reload. 
  Popups.attach(context, '.popups-form-noupdate', {noUpdate: true});  // no reload at all.
  Popups.attach(context, '.popups-form-noreload', {noUpdate: true});  // Obsolete.
};

/*
 
// ***************************************************************************
// Popup Namespace
// ***************************************************************************

// Create the popup class.
// A popup up is a single modal dialog.
Popup = function() {
  this.id = 'popups-' + Popups.nextCounter();
};
Popup.prototype.$popup = function() {
  return $('#' + this.id);
}
Popup.prototype.$popupBody = function() {
  return $('#' + this.id + ' .popups-body');
}
Popup.prototype.$popupClose = function() {
  return $('#' + this.id + ' .popups-close');
}
Popup.prototype.$popupTitle = function() {
  return $('#' + this.id + ' .popups-title');
}
Popup.prototype.$popupButtons = function() {
  return $('#' + this.id + ' .popups-buttons');
}
Popup.prototype.$popupFooter = function() {
  return $('#' + this.id + ' .popups-footer');
}

Popup.prototype.hide = function() {
  this.$popup().css('left', '-9999px');
};

Popup.prototype.show = function() {
  Popups.resizeAndCenter(this);
};

Popup.prototype.open = function(title, body, buttons, width){
  return Popups.open(this, title, body, buttons, width);
};

Popup.prototype.removePopup = function() { 
  Popups.removePopup(this);
}; 

// Remove everything.
Popup.prototype.close = function() {
  return Popups.close(this);
};

// Set the focus on the popups to the first visible, enabled form element, or the close link.
Popup.prototype.refocus = function() {
  // Select the first visible enabled input element.
  var $popup = this.$popup();
  var $focus = $popup.find(':input:visible:enabled:first');
  if (!$focus.length) {
    // There is no visible enabled input element, so select the close link.
    $focus = $popup.find('.popups-close a');
  }
  $focus.focus();
};
*/

// ***************************************************************************
// POPUPS Namespace
// ***************************************************************************

/**
 * Create the popups namespace.
 * The popups namespace contains a set of functions for managing multiple popup objects.
 */
Popups = function(){};

Popups.popupStack = []; 
Popups.originalSettings = null;
Popups.newCss = [];

/**
 * Create the popup class.
 * A popup up is a single modal dialog.
 */
Popups.Popup = function() {
  this.id = 'popups-' + Popups.nextCounter();
};
Popups.Popup.prototype.$popup = function() {
  return $('#' + this.id);
};
Popups.Popup.prototype.$popupBody = function() {
  return $('#' + this.id + ' .popups-body');
};
Popups.Popup.prototype.$popupClose = function() {
  return $('#' + this.id + ' .popups-close');
};
Popups.Popup.prototype.$popupTitle = function() {
  return $('#' + this.id + ' .popups-title');
};
Popups.Popup.prototype.$popupButtons = function() {
  return $('#' + this.id + ' .popups-buttons');
};
Popups.Popup.prototype.$popupFooter = function() {
  return $('#' + this.id + ' .popups-footer');
};

Popups.Popup.prototype.hide = function() {
  this.$popup().css('left', '-9999px');
};

Popups.Popup.prototype.show = function() {
  Popups.resizeAndCenter(this);
};

Popups.Popup.prototype.open = function(title, body, buttons, width){
  return Popups.open(this, title, body, buttons, width);
};

Popups.Popup.prototype.removePopup = function() { 
  Popups.removePopup(this);
}; 

/**
 * Remove everything.
 */
Popups.Popup.prototype.close = function() {
  return Popups.close(this);
};

/**
 * Set the focus on the popups to the first visible, enabled form element, or the close link.
 */
Popups.Popup.prototype.refocus = function() {
  // Select the first visible enabled input element.
  var $popup = this.$popup();
  var $focus = $popup.find(':input:visible:enabled:first');
  if (!$focus.length) {
    // There is no visible enabled input element, so select the close link.
    $focus = $popup.find('.popups-close a');
  }
  $focus.focus();
};

// Utility function.
Popups.isset = function(v) {
  return (typeof(v) !== 'undefined');
};

Popups.activePopup = function() {
  if (Popups.popupStack.length) {
    return Popups.popupStack[Popups.popupStack.length - 1]; // top of stack.
  }
  else {
    return null;
  }
};

Popups.push = function(popup) {
  Popups.popupStack.push(popup);
};

// Should I integrate this with popupRemove??
Popups.pop = function(popup) {
  return Popups.popupStack.pop();
};

/**
 * Attach the popups behavior to a particular link.
 *
 * @param selector
 *   jQuery selector for links to attach popups behavior to.
 * @param options
 *   Hash of options associated with these links.
 */
Popups.attach = function(context, selector, options) {
  $(selector, context).not('.popups-processed').each(function() {
    var $element = $(this);  
    
    // Mark the element as attached.    
    var title = $element.attr('title') || '';
    $element.attr('title', title + ' ' + Drupal.t('[Popup]') + ' '); // Append note to link title.
    $element.addClass('popups-processed');
    
    // Attach the on-click popup behavior to the element.
    $element.click(function(e){
      // TODO - break this into seperate function. 
      var element = this;

      // If element is inside of a #popup div, show alert and bail out. 
//      if ($(element).parents('form').parents('#popups').length) { 
//      if ($(element).parents('form').parents('.popups').length) { 
//        var go = confirm("Opening a new popup here will discard your current form input.\n" +
//                         "Click 'OK' to open new popup and discard current form.\n" +
//                         "Click 'Cancel' to stay in current form.");
//        if (!go) return false;
//      }

      // If the element contains a on-popups-options attribute, use it instead of options param.
      if ($(element).attr('on-popups-options')) {
//        options = eval('(' + $(element).attr('on-popups-options') + ')'); 
        options = Drupal.parseJson($(element).attr('on-popups-options')); 
      }
			
      // If the option is distructive, check if the page is already modified, and offer to save.
      var pageIsDirty = $('span.tabledrag-changed').size() > 0;
      var willModifyOriginal = !options.noUpdate;
      if (pageIsDirty && willModifyOriginal) {
        // The user will lose modifications, so show dialog offering to save current state.
        var popup = new Popups.Popup();
        var body = Drupal.t("There are unsaved changes on this page, which you will lose if you continue.");
        var buttons = {
         'popup_save': {title: Drupal.t('Save Changes'), func: function(){Popups.savePage(element, options);}},
         'popup_submit': {title: Drupal.t('Continue'), func: function(){popup.removePopup(); Popups.openPath(element, options);}},
         'popup_cancel': {title: Drupal.t('Cancel'), func: function(){popup.close();}}
        };
        popup.open(Drupal.t('Warning: Please Confirm'), body, buttons);
      }
      else {
        // Page is clean, or popup is safe, so just open it.
        Popups.openPath(element, options);
      }
      return false; 
    });    
  });
};

/**
 * Generic dialog builder.
 * TODO: capture the focus when it leaves the dialog.
 */
Popups.open = function(popup, title, body, buttons, width){
  Popups.addOverlay();
  
  if (Popups.activePopup()) {
//    console.log("Hiding active popup");
    Popups.activePopup().hide();
  }
  
  if (!popup) {
    popup = new Popups.Popup();
  }
//  console.log(popup);
//  Popups.activePopup = popup;
  Popups.push(popup);
  
  var $popup = $(Drupal.theme('popupDialog', popup.id, title, body, buttons));
  // Start with dialog off the side. Making it invisible causes flash in FF2.
  $popup.css('left', '-9999px');
  if (width) {
    $popup.css('width', width);
  }
  $('body').append($popup); // Add the popup to the DOM.
  // Add button function callbacks.
  if (buttons) {
    jQuery.each(buttons, function(id, button){
      $('#' + id).click(button.func);
    });
  }
  
  //  $('#popups-close').click(Popups.close);
  //  $('a.popups-close').click(Popups.close);
  //  var popup = this;
  
  // Add the default click-to-close behavior.
  popup.$popupClose().click(function(){
    return Popups.close(popup);
  });

  Popups.resizeAndCenter(popup);

  // Focus on the first input element in the popup window.
  popup.refocus(); 
  
  // Remove the loading image.
  Popups.removeLoading();
   
  return popup;
// TODO  return false;
};  

Popups.resizeAndCenter = function(popup) {
//  console.log('resizeAndCenter: ' + popup.id)
  
  var $popup = popup.$popup();
  
  // center on the screen, adding in offsets if the window has been scrolled
  var popupWidth = $popup.width();  
  var windowWidth = Popups.windowWidth();
  var left = (windowWidth / 2) - (popupWidth / 2) + Popups.scrollLeft();
  
  // Get popups's height on the page.
  $popup.css('height', 'auto'); // Reset height.
  var popupHeight = $popup.height(); 
  $popup.height(popupHeight);
  var windowHeight = Popups.windowHeight();
   
  if (popupHeight > (0.9 * windowHeight) ) { // Must fit in 90% of window.
    popupHeight = 0.9 * windowHeight;
    $popup.height(popupHeight);
  }  
  var top = (windowHeight / 2) - (popupHeight / 2) + Popups.scrollTop();

  $popup.css('top', top).css('left', left); // Position the popups to be visible. 
};
  

/**
 *  Simple popup dialog that functions like the browser's alert box.
 */
Popups.message = function(title, message) {
  message = message || '';
  var popup = new Popups.Popup();
  var buttons = {
    'popup_ok': {title: Drupal.t('OK'), func: function(){popup.close();}}
  };
  popup.open(title, message, buttons);
  return popup;
};

/**
 * Handle any special keys when popups is active.
 */
Popups.keyHandle = function(e) {
  if (!e) {
    e = window.event;
  }
  switch (e.keyCode) {
    case 27: // esc
      Popups.close();
      break;
    case 191: // '?' key, show help.
      if (e.shiftKey && e.ctrlKey) {
        var $help = $('a.popups.more-help');
        if ($help.size()) {
          $help.click();
        }
        else {
          Popups.message(Drupal.t("Sorry, there is no additional help for this page"));
        }
      }
      break;
  }
};

/*****************************************************************************
 * Appearence Functions (overlay, loading graphic, remove popups)     *********
 *****************************************************************************/
 
/**
 * Add full page div between the page and the dialog, to make the popup modal.
 */
Popups.addOverlay = function() {
  var $overlay = $('#popups-overlay');
  if (!$overlay.length) { // Overlay does not already exist, so create it.
    $overlay = $(Drupal.theme('popupOverlay'));
    $overlay.css('opacity', '0.4'); // for ie6(?)
    // Doing absolute positioning, so make overlay's size equal the entire body.
    var $doc = $(document);
    $overlay.width($doc.width()).height($doc.height()); 
    $overlay.click(function(){Popups.close();});
    $('body').prepend($overlay);
  }
};

/**
 * Remove overlay if popupStack is empty.
 */
Popups.removeOverlay = function() {
  if (!Popups.popupStack.length) {
    $('#popups-overlay').remove();
  }
};

/**
 * Add a "Loading" message while we are waiting for the ajax response.
 */
Popups.addLoading = function() {
  var $loading = $('#popups-loading');
  if (!$loading.length) { // Loading image does not already exist, so create it.
    $loading = $(Drupal.theme('popupLoading'));
    $('body').prepend($loading); // Loading div is initially display:none.
    var width = $loading.width();
    var height = $loading.height();
    var left = (Popups.windowWidth() / 2) - (width / 2) + Popups.scrollLeft();
    var top = (Popups.windowHeight() / 2) - (height / 2) + Popups.scrollTop();
    $loading.css({'top': top, 'left': left, 'display': 'block'}); // Center it and make it visible.
  }
};

Popups.removeLoading = function() {
  $('#popups-loading').remove();
};

// Should I fold this function into Popups.pop?
Popups.removePopup = function(popup) {  
//  console.log("Popups.removePopup: " + popup);
  if (!Popups.isset(popup)) {
    popup = Popups.activePopup();
  }
  if (popup) {
//    console.log('removing '+popup.id);
    popup.$popup().remove();
    Popups.popupStack.splice(Popups.popupStack.indexOf(popup), 1); // Remove popup from stack.  Probably should rework into .pop()
  }  
//  else {
//    console.log("Popups.removePopup - there is no popup to remove.");
//  }
}; 

/**
 * Remove everything.
 */
Popups.close = function(popup) {
  if (!Popups.isset(popup)) {
    popup = Popups.activePopup();
  }
  Popups.removePopup(popup);  // Should this be a pop??
  Popups.removeLoading();
  if (Popups.activePopup()) {
    Popups.activePopup().show();
    Popups.activePopup().refocus();
  }
  else {
    Popups.removeOverlay();
    Popups.restorePage();
  }
  return false;
};

/**
 * Remove as much of the effects of jit loading as possible.
 */
Popups.restorePage = function() {
  // Restore the page's original Drupal.settings.
  if (Popups.originalSettings) {
//    console.log('restoring page');
    Drupal.settings = Popups.originalSettings;
    // Remove the CSS files that were jit loaded for popup.
    for (var i in Popups.newCss) {
      var link = Popups.newCss[i];
      $('link[href='+ $(link).attr('href') + ']').remove();
    }
    Popups.originalSettings = null;
    Popups.newCss = [];
  } 
};


/****************************************************************************
 * Utility Functions   ******************************************************
 ****************************************************************************/

/**
 * Get the position of the left side of the browser window.
 */
Popups.scrollLeft = function() {
  return Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
};

/**
 * Get the position of the top of the browser window.
 */
Popups.scrollTop = function() {
  return Math.max(document.documentElement.scrollTop, document.body.scrollTop);
};

/**
 * Get the height of the browser window.
 * Fixes jQuery & Opera bug - http://drupal.org/node/366093
 */
Popups.windowHeight = function() {
  if ($.browser.opera && $.browser.version > "9.5" && $.fn.jquery <= "1.2.6") { 
//    return document.documentElement["clientHeight"];
    return document.documentElement.clientHeight;
  }
  return $(window).height();
};

/**
 * Get the height of the browser window.
 * Fixes jQuery & Opera bug - http://drupal.org/node/366093
 */
Popups.windowWidth = function() {
  if ($.browser.opera && $.browser.version > "9.5" && $.fn.jquery <= "1.2.6") { 
//    return document.documentElement["clientWidth"];
    return document.documentElement.clientWidth;
  }
  return $(window).width();
};

Popups.nextCounter = function() {
  if (this.counter === undefined) {
    this.counter = 0;
  }
  else {
    this.counter++;
  }
  return this.counter;
};

/****************************************************************************
 * Ajax Functions   ******************************************************
 ****************************************************************************/

/**
 * Add additional CSS to the page.
 */
Popups.addCSS = function(css) {
  Popups.newCss = [];
  for (var type in css) {
    for (var file in css[type]) {
      var link = css[type][file];
      // Does the page already contain this stylesheet?
      if (!$('link[href='+ $(link).attr('href') + ']').length) {
//        console.log( "ADDING " + link);
        $('head').append(link);
        Popups.newCss.push(link); // Keep a list, so we can remove them later.
      }
    }
  }
};

/**
 * Add additional Javascript to the page.
 */
Popups.addJS = function(js) {
  var scripts = [];
  var inlines = [];
  for (var type in js) {
    if (type != 'setting') {
      for (var file in js[type]) {
        if (type == 'inline') {
          inlines.push($(js[type][file]).text());
        }
        else {
          scripts.push($(js[type][file]).attr('src'));
        }
      }
    }
  }

  // Add new JS settings to the page, needed for #ahah properties to work.
  Popups.originalSettings = Drupal.settings; // Cache orginial page settings.
  Drupal.settings = js.setting;
  // As an exception, keep the popups settings in place until the popup is closed.
  Drupal.settings.popups = Popups.originalSettings.popups;

  for (var i in scripts) {
    var src = scripts[i];
    if (!$('script[src='+ src + ']').length) {
      // Get the script from the server and execute it.
      $.ajax({ 
        type: 'GET',
        url: src,
        dataType: 'script',
        async : false,
        success: function(script) {
          eval(script);
        }
      });
    }
  }

  return inlines;
};

/**
 * Execute the jit loaded inline scripts.
 * Q: Do we want to re-excute the ones already in the page?
 * 
 * @param inlines
 *   Array of inline scripts.
 */
Popups.addInlineJS = function(inlines) {
  // Load the inlines into the page.
  for (var n in inlines) {
    // If the script is not already in the page, execute it.
    if (!$('script:not([src]):contains(' + inlines[n] + ')').length) {
      eval(inlines[n]);
    }
  }
};

Popups.beforeSend = function(xhr) {
  xhr.setRequestHeader("X-Drupal-Render-Mode", 'json/popups');
};

/**
 * Do before the form in the popups is submitted.
 */
Popups.beforeSubmit = function(formData, $form, options) {
  Popups.removePopup(); // Remove just the dialog, but not the overlay.
  Popups.addLoading();
};


/****************************************************************************
 * Page & Form in popups functions                                         ***
 ****************************************************************************/

/**
 * Use Ajax to open the link in a popups window.
 *
 * @param element
 *   Element that was clicked to open the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 */
Popups.openPath = function(element, options) {
  // let the user know something is happening
  $('body').css("cursor", "wait");
  
  // TODO - get nonmodal working.
  if (!options.nonModal) {
    Popups.addOverlay(); 
  }
  Popups.addLoading();
  
  var href = options.href ? options.href : element.href;
  $(document).trigger('popups_open_path', [element, href]); // Broadcast Popup Open Path event.
  
  var params = {}; 
  // Force the popups to return back to the orignal page when forms are done, unless forceReturn option is set.
  if (!options.forceReturn) { // If forceReturn, requestor wants data from different page.
    href = href.replace(/destination=[^;&]*[;&]?/, ''); // Strip out any existing destination param.
    params.destination = Drupal.settings.popups.originalPath; // Set the destination to the original page.    
  }

  var ajaxOptions = {
    url: href,
    dataType: 'json',
    data: params,
    beforeSend: Popups.beforeSend,
    success: function(json) { 
      // Add additional CSS to the page.
      Popups.addCSS(json.css);
      var inlines = Popups.addJS(json.js);    
      var popup = Popups.openContent(json.title, json.messages + json.content, options, element);
      Popups.addInlineJS(inlines);   
      // Broadcast an event that the path was opened.
//      console.log("Sending popups_path_opened");
      $(document).trigger('popups_open_path_done', [element, href, popup]);   
    },
    complete: function() {
      $('body').css("cursor", "auto"); // Return the cursor to normal state.      
    }
  };

  if (options.reloadOnError) {
    ajaxOptions.error = function() {
      location.reload(); // Reload on error. Is this working?
    };    
  }
  else {
    ajaxOptions.error = function() {
      Popups.message("Unable to open: " + href);
    };
  }
  $.ajax(ajaxOptions);
        
  return false;         
};

/**
 * Open content in an ajax popups.
 *
 * @param title
 *   String title of the popups.
 * @param content
 *   HTML to show in the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 * @param element
 *   A DOM object containing the element that was clicked to initiate the popup. 
 */
Popups.openContent = function(title, content, options, element) {
  var popup = Popups.open(null, title, content, null, options.width); 
  // Add behaviors to content in popups. 
  delete Drupal.behaviors.tableHeader; // Work-around for bug in tableheader.js (http://drupal.org/node/234377)
  delete Drupal.behaviors.teaser; // Work-around for bug in teaser.js (sigh).
//  Drupal.attachBehaviors($('#popups-body'));
  Drupal.attachBehaviors(popup.$popupBody());
  // Adding collapse moves focus.
  popup.refocus();

  // If the popups contains a form, capture submits.
//  var $form = $('form', '#popups-body');
//  var $form = $('form', popup.id + ' .popups-body');
  var $form = $('form', popup.$popupBody());
  $form.ajaxForm({   
    dataType: 'json',   
    beforeSubmit: Popups.beforeSubmit,
    beforeSend: Popups.beforeSend,
    success: function(response, status) {
      Popups.formSuccess(response, options, element);
    },
    error: function() {
      Popups.message("Bad Response form submission");
    }
  });
  return popup;
};

/**
 * The form in the popups was successfully submitted
 * Update the originating page.
 * Show any messages in a popups.
 * 
 * @param response
 *   JSON object from server with status of form submission.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 *     noUpdate: bool, does the popups effect the underlying page.
 *     nonModal: bool, does the popups block access to the underlying page.
 *     targetSelectors: hash of jQuery selectors, overrides defaultTargetSelector.
 *     titleSelectors: array of jQuery selectors, where to put the the new title of the page.
 *     noMessage: don't show final message in popup.
 * @param element
 *   A DOM object containing the element that was clicked to initiate the popup. 
 */
Popups.formSuccess = function(data, options, element) {  
  // Determine if we are at an end point, or just moving from one popups to another.
  var done = (data.path === Drupal.settings.popups.originalPath) || (data.path === options.forceReturn);
  if (!done) { // Not done yet, so show new page in new popups.
    Popups.removeLoading();
    Popups.openContent(data.title, data.messages + data.content, options, element);
  }
  else { // Done.
    // Execute the afterSubmit callback if available.
    if (options.afterSubmit) { 
      var afterSubmitResult = eval(options.afterSubmit +'(data, options, element)');
      if (afterSubmitResult === false) { // Give afterSubmit callback a chance to skip normal processing.
        return;
      }
    }

    if (options.reloadWhenDone) { // Force a complete, non-ajax reload of the page.
      location.reload(); 
    }
    else { // Normal, targeted ajax, reload behavior.
      // show messages in dialog and embed the results in the original page.
      var showMessage = data.messages.length && !options.noMessage;
      if (showMessage) {
        var popup = Popups.message(data.messages);
//        console.log("Message: " + popup)
        if (Drupal.settings.popups.autoCloseFinalMessage) {
          setTimeout(function(){Popups.close(popup);}, 2500); // Autoclose the message box in 2.5 seconds.
        }
  
        // Insert the message into the page above the content.
        // Might not be the standard spot, but it is the easiest to find.
        var $next = $(Drupal.settings.popups.defaultTargetSelector);
        $next.parent().find('div.messages').remove(); // Remove the current messages.
        $next.before(data.messages);
      }
          
      // Update the content area (defined by 'targetSelectors').
      if (!options.noUpdate) { 
        Popups.testContentSelector();
        if (Popups.isset(options.targetSelectors)) { // Pick and choose what returned content goes where.
          jQuery.each(options.targetSelectors, function(t_new, t_old) {
            if(!isNaN(t_new)) {
              t_new = t_old; // handle case where targetSelectors is an array, not a hash.
            }
            var new_content = $(t_new, data.content);
            var $c = $(t_old).html(new_content); // Inject the new content into the original page.
            Drupal.attachBehaviors($c);  
          });
        }
        else { // Put the entire new content into default content area.
          var $c = $(Drupal.settings.popups.defaultTargetSelector).html(data.content);
          Drupal.attachBehaviors($c);                    
        }
      }
      
      // Update the title of the page.
      if (Popups.isset(options.titleSelectors)) {
        jQuery.each(options.titleSelectors, function() {
          $(''+this).html(data.title);
        });
      }
              
      // Done with changes to the original page, remove effects.
      Popups.removeLoading();
      if (!showMessage) { 
        // If there is not a messages popups, close current layer.
        Popups.close();
      }
    }
    
    // Broadcast an event that popup form was done and successful.
    $(document).trigger('popups_form_success', [element, popup]);
    
  }  // End of updating original page.
}; 

/**
 * Submit the page and reload the results, before popping up the real dialog.
 *
 * @param element
 *   Element that was clicked to open the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 */
Popups.savePage = function(popup, element, options) {
  var target = Drupal.settings.popups.defaultTargetSelector;
  var $form = $('form', target);
  var ajaxOptions = {
    dataType: 'json',
    beforeSubmit: Popups.beforeSubmit,   
    beforeSend: Popups.beforeSend,
    success: function(response, status) { 
      // Sync up the current page contents with the submit.
      var $c = $(target).html(response.content); // Inject the new content into the page.
      Drupal.attachBehaviors($c);
      // The form has been saved, the page reloaded, now safe to show the link in a popup.
      Popups.openPath(popup, element, options); 
    } 
  };
  $form.ajaxSubmit(ajaxOptions); // Submit the form. 
};

/**
 * Warn the user if ajax updates will not work
 *   due to mismatch between the theme and the theme's popup setting.
 */
Popups.testContentSelector = function() {
  var target = Drupal.settings.popups.defaultTargetSelector;
  var hits = $(target).length;
  if (hits !== 1) { // 1 is the corrent answer.
    var msg = Drupal.t('The popup content area for this theme is misconfigured.') + '\n';
    if (hits === 0) {
      msg += Drupal.t('There is no element that matches ') + '"' + target + '"\n';
    }
    else if (hits > 1) {
      msg += Drupal.t('There are multiple elements that match: ') + '"' + target + '"\n';
    }
    msg += Drupal.t('Go to admin/build/themes/settings, select your theme, and edit the "Content Selector" field'); 
    alert(msg);
  }
};


/****************************************************************************
 * Theme Functions   ********************************************************
 ****************************************************************************/

Drupal.theme.prototype.popupLoading = function() {
  var loading = '<div id="popups-loading">';
  loading += '<img src="'+ Drupal.settings.basePath + Drupal.settings.popups.modulePath + '/ajax-loader.gif" />';
  loading += '</div>';
  return loading;
};

Drupal.theme.prototype.popupOverlay = function() {
  return '<div id="popups-overlay"></div>';
};

Drupal.theme.prototype.popupButton = function(title, id) {
  return '<input type="button" value="'+ title +'" id="'+ id +'" />';
};

Drupal.theme.prototype.popupDialog = function(popupId, title, body, buttons) {
  var template = Drupal.theme('popupTemplate', popupId);
  var popups = template.replace('%title', title).replace('%body', body);
  
  var themedButtons = '';
  if (buttons) {
    jQuery.each(buttons, function (id, button) { 
      themedButtons += Drupal.theme('popupButton', button.title, id);
    });  
  }  
  popups = popups.replace('%buttons', themedButtons);  
  return popups;
};

Drupal.theme.prototype.popupTemplate = function(popupId) {
  var template;
  template += '<div id="'+ popupId + '" class="popups-box">';
  template += '  <div class="popups-title">';
  template += '    <div class="popups-close"><a href="#">' + Drupal.t('Close') + '</a></div>';
  template += '    <div class="title">%title</div>';
  template += '    <div class="clear-block"></div>';
  template += '  </div>';
  template += '  <div class="popups-body">%body</div>';
  template += '  <div class="popups-buttons">%buttons</div>';
  template += '  <div class="popups-footer"></div>';
  template += '</div>';
  return template;
};

